from zc.relation.interfaces import ICatalog
from zope.component import getGlobalSiteManager
from zope.component import provideUtility
from zope.interface import implementer
from zope.intid.interfaces import IIntIds

import z3c.relationfield


@implementer(IIntIds)
class MockIntIds(object):
    """Dumb utility for unit tests, returns sequential integers. Not a
    complete implementation."""

    def getId(self, ob):
        """Appropriately raises KeyErrors when the object is not registered,
        e.g. always"""
        raise KeyError(ob)


mock_intids = MockIntIds()


@implementer(ICatalog)
class MockCatalog(object):
    """Does nothing except exist"""

    def findRelations(self, query):
        return []


mock_catalog = MockCatalog()


def register_fake_intid():
    provideUtility(mock_intids)


def unregister_fake_intid():
    sm = getGlobalSiteManager()
    sm.unregisterUtility(mock_intids)


def register_fake_catalog():
    provideUtility(mock_catalog)


def unregister_fake_catalog():
    sm = getGlobalSiteManager()
    sm.unregisterUtility(mock_catalog)


@implementer(z3c.relationfield.interfaces.IHasRelations)
class MockContent(object):
    pass



from unittest import makeSuite, TestCase, TestSuite
from z3c.relationfield.testing import (
    register_fake_intid,
    register_fake_catalog,
    unregister_fake_intid,
    unregister_fake_catalog,
    MockContent,
)
from z3c.relationfield import event
from z3c.relationfield import RelationList, Relation
from zope.component.interfaces import ObjectEvent
from zope.component.interfaces import ComponentLookupError


class FieldTests(TestCase):
    """Unit tests for fields
    """

    def test_list_value_type(self):
        f = RelationList(title=u"Test")
        self.failUnless(isinstance(f.value_type, Relation))


class EventTests(TestCase):
    """Unit tests for the relation field event handlers.  The event handlers
    should be tolerant of missing utilities and objects not yet registered with
    the intid utility."""

    def setUp(self):
        self.content = MockContent()
        register_fake_catalog()
        register_fake_intid()

    def tearDown(self):
        unregister_fake_catalog()
        unregister_fake_intid()

    def test_missing_intids(self):
        """Event handlers which trigger on object removal should not
        throw exceptions when the utilities are missing.  The utilities may
        have been deleted in the same transaction (e.g. site deletion)."""
        # Remove intid utility and ensure the event handler doesn't fail
        unregister_fake_intid()
        try:
            event.breakRelations(ObjectEvent(self.content))
        except ComponentLookupError:
            self.fail("breakRelations fails when intid utility is missing")

    def test_break_relations_missing_catalog(self):
        # Remove relations catalog and ensure the event handler doesn't fail
        unregister_fake_catalog()
        try:
            event.breakRelations(ObjectEvent(self.content))
        except ComponentLookupError:
            self.fail("breakRelations fails when catalog utility is missing")

    def test_remove_relations_missing_catalog(self):
        # Remove relations catalog and ensure the event handler doesn't fail
        unregister_fake_catalog()
        try:
            event.removeRelations(self.content, None)
        except ComponentLookupError:
            self.fail("removeRelations fails when catalog utility is missing")

    def test_initid_failure(self):
        """When an object has not yet been registered with the intid utility,
        getId fails with a KeyError.  The event handlers need to handle this
        cleanly."""
        # Our content is not yet registered with the intid utility
        # and has no __parent__ attribute
        try:
            event.updateRelations(self.content, None)
        except (KeyError, AttributeError):
            self.fail("updateRelations fails with unregistered object")


def test_suite():
    suite = TestSuite()
    suite.addTest(makeSuite(EventTests))
    suite.addTest(makeSuite(FieldTests))
    return suite



from zope.component.testlayer import ZCMLFileLayer
import doctest
import re
import six
import sys
import unittest


# Evil hack to make pickling work with classes defined in doc tests
class NoCopyDict(dict):
    def copy(self):
        return self


class Py23DocChecker(doctest.OutputChecker):

    def check_output(self, want, got, optionflags):
        if six.PY2:
            want = re.sub("b'(.*?)'", "'\\1'", want)
        else:
            want = re.sub("u'(.*?)'", "'\\1'", want)

        return doctest.OutputChecker.check_output(self, want, got, optionflags)


class FakeModule:
    """A fake module."""

    def __init__(self, dict):
        self.__dict = dict

    def __getattr__(self, name):
        try:
            return self.__dict[name]
        except KeyError:
            raise AttributeError(name)


def setUp(test):
    test.globs['__name__'] = '__builtin__'
    test.globs = NoCopyDict(test.globs)
    sys.modules['__builtin__'] = FakeModule(test.globs)


def tearDown(test):
    del sys.modules['__builtin__']
    test.globs.clear()


def test_suite():
    globs = {}
    readme = doctest.DocFileSuite(
        'README.rst',
        globs=globs,
        setUp=setUp,
        tearDown=tearDown,
        optionflags=doctest.ELLIPSIS,
        checker=Py23DocChecker(),
    )
    import z3c.relationfield
    readme.layer = ZCMLFileLayer(z3c.relationfield)
    return unittest.TestSuite([readme])



from z3c.relationfield.interfaces import IRelation
from z3c.relationfield.interfaces import IRelationChoice
from z3c.relationfield.interfaces import IRelationList
from zope.interface import implementer
from zope.schema import Choice
from zope.schema import Field
from zope.schema import List


@implementer(IRelation)
class Relation(Field):
    pass


@implementer(IRelationList)
class RelationList(List):

    def __init__(self, value_type=None, unique=False, **kw):
        if value_type is None:
            value_type = Relation()
        super(RelationList, self).__init__(
            value_type=value_type,
            unique=unique,
            **kw
        )


@implementer(IRelationChoice)
class RelationChoice(Choice):
    pass
